#ifndef __bbcreaMaracasVisuContourToControlPoints_h_INCLUDED__
#define __bbcreaMaracasVisuContourToControlPoints_h_INCLUDED__
#include "bbtkAtomicBlackBox.h"
#include "iostream"

#include "Contour/ExtractControlPoints2D.h"
#include "Contour/ContourPropagation.h"
#include "Contour/Propagation.h"

#include "vtkImageData.h"

namespace bbcreaMaracasVisu
{

class /*BBTK_EXPORT*/ ContourToControlPoints
 : 
   public bbtk::AtomicBlackBox
{
	//BBTK_USER_BLACK_BOX_INTERFACE(ContourToControlPoints,bbtk::AtomicBlackBox);
	BBTK_BLACK_BOX_INTERFACE(ContourToControlPoints,bbtk::AtomicBlackBox);
  BBTK_DECLARE_INPUT(In,double);
		BBTK_DECLARE_INPUT(Sampling, double);
		BBTK_DECLARE_INPUT(Slice,int);
		BBTK_DECLARE_INPUT(InContourX, std::vector<double>*);
		BBTK_DECLARE_INPUT(InContourY, std::vector<double>*);
		BBTK_DECLARE_INPUT(InContourZ, std::vector<double>*);
		BBTK_DECLARE_INPUT(RadRBF, double);
		BBTK_DECLARE_INPUT(Option, int);
		BBTK_DECLARE_INPUT(LoadContour, int);
  BBTK_DECLARE_OUTPUT(Out,double);
		BBTK_DECLARE_OUTPUT(ControlPointX, std::vector<double>*);
		BBTK_DECLARE_OUTPUT(ControlPointY, std::vector<double>*);
		BBTK_DECLARE_OUTPUT(ControlPointZ, std::vector<double>*);
		BBTK_DECLARE_OUTPUT(ImagePropg, vtkImageData*);

  BBTK_PROCESS(Process);
  void Process();
  private:
			int pos;
			bool inic;
			
			ContourPropagation		*_contprop;
			PropContour				*_propgdata;
			ExtractControlPoints2D	*_ext2D;

			vtkImageData			*_imagedata;
			FILE					*_fd;
	  		
			std::vector<double> OutContourX;
			std::vector<double> OutContourY;
			std::vector<double> OutContourZ;

			std::vector<double> KeyContourX;
			std::vector<double> KeyContourY;
			std::vector<double> KeyContourZ;
			std::vector<int> KeySizes;
			
			//TEMP DATA
			std::vector<double> tempX;
			std::vector<double> tempY;
			std::vector<double> tempZ;
			std::vector<int>::iterator it;
  
};

BBTK_BEGIN_DESCRIBE_BLACK_BOX(ContourToControlPoints,bbtk::AtomicBlackBox);
BBTK_NAME("ContourToControlPoints");
BBTK_AUTHOR("Juan Sebasltian Torres");
BBTK_DESCRIPTION("Generates the control points of a contour");
BBTK_CATEGORY("filter");
   BBTK_INPUT(ContourToControlPoints,Sampling,"Sampling (in %) to obtain the control points",double,"");
   BBTK_INPUT(ContourToControlPoints,Slice,"Slice Number Propagation",int,"");
   BBTK_INPUT(ContourToControlPoints,InContourX,"Coordinate X of a Contour",std::vector<double>*,"");
   BBTK_INPUT(ContourToControlPoints,InContourY,"Coordinate Y of a Contour",std::vector<double>*,"");
   BBTK_INPUT(ContourToControlPoints,InContourZ,"Coordinate Z of a Contour",std::vector<double>*,"");
   BBTK_INPUT(ContourToControlPoints,RadRBF,"Radius of th RBF method",double,"");
   BBTK_INPUT(ContourToControlPoints,Option,"1 -> Control Points || 2 -> Propagation",int,"");
   BBTK_INPUT(ContourToControlPoints,LoadContour,"1 -> Load Saved Manual Points",int,"");
//THE OUTPUTS
   BBTK_OUTPUT(ContourToControlPoints,ControlPointX,"Processed Coordinate X of the Boundary",std::vector<double>*,"");
   BBTK_OUTPUT(ContourToControlPoints,ControlPointY,"Processed Coordinate Y of the Boundary",std::vector<double>*,"");
   BBTK_OUTPUT(ContourToControlPoints,ControlPointZ,"Processed Coordinate Z of the Boundary",std::vector<double>*,"");
   BBTK_OUTPUT(ContourToControlPoints,ImagePropg,"vtkImageData of the propagation image-3D",vtkImageData*,"");
BBTK_END_DESCRIBE_BLACK_BOX(ContourToControlPoints);
}
// EO namespace bbcreaMaracasVisu

#endif // __bbcreaMaracasVisuContourToControlPoints_h_INCLUDED__

