/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkInteractorStyleCutter.h,v $
  Language:  C++
  Date:      $Date: 2009/05/14 13:54:57 $
  Version:   $Revision: 1.1 $

  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkInteractorStyleCutter - 
// .SECTION Description
// This interactor style allows the user to draw a rectangle in the render
// window using the left mouse button.  

#ifndef __vtkInteractorStyleCutter_h
#define __vtkInteractorStyleCutter_h

#include <vtkInteractorStyle.h>

class vtkPoints;
class vtkActor2D;
class vtkCellArray;

class vtkInteractorStyleCutter : public vtkInteractorStyle
{
public:
  // Description:
  // Instantiate the object.
  static vtkInteractorStyleCutter *New();
  vtkTypeRevisionMacro(vtkInteractorStyleCutter, vtkInteractorStyle);
  void PrintSelf(ostream& os, vtkIndent indent);
  
  // Description:
  // Event bindings controlling the effects of pressing mouse buttons
  // or moving the mouse.
  virtual void OnMouseMove();
  virtual void OnLeftButtonDown();
  virtual void OnRightButtonDown();
  void VisibilityOff();
  
  vtkGetObjectMacro(LoopPoints,vtkPoints);
  vtkGetVector3Macro(Direction,double);
  
protected:
  vtkInteractorStyleCutter();
  ~vtkInteractorStyleCutter();

  //This method is designed for vtkImplicitSelectionLoop where the loop
  //should not be closed, the function is virtual to override this behavior
  virtual void EndLoop();
  void Initialize();

  int CurrentPosition[2];
  double Direction[3];

  //Points selected by the user
  vtkPoints *Points;

  //Current point id:
  vtkIdType PointID;

  //bool which determine if user has press the left button first
  int Moving;
  
  vtkActor2D	*BboxActor;
  vtkCellArray	*Lines;
  vtkPoints		*LoopPoints;
  
private:
  vtkInteractorStyleCutter(const vtkInteractorStyleCutter&);  // Not implemented
  void operator=(const vtkInteractorStyleCutter&);  // Not implemented
};

#endif
