
#include "wxMaracasMultipleVolumeRendererManagerData.h"

/********************************************************************************************
** Start of data viewmanagerData
*********************************************************************************************/

wxMaracasMultipleVolumeRendererManagerData::wxMaracasMultipleVolumeRendererManagerData(vtkImageData* vol, std::string dataname){

	_vol = vol;	
	_dataname = dataname;
	
	_tfun = vtkPiecewiseFunction::New();
	_ctfun = vtkColorTransferFunction::New();   

	_volumePlanes  = vtkPlanes::New();
	_compositeFunction = vtkVolumeRayCastCompositeFunction::New();
	_volumeMapper = vtkVolumeRayCastMapper::New();	
	_volumeMapper->SetVolumeRayCastFunction(_compositeFunction);
	_volumeMapper->SetClippingPlanes( _volumePlanes );
	_volumeMapper->AutoAdjustSampleDistancesOn();
	_volumeProperty = vtkVolumeProperty::New();	
	_volumeProperty->SetInterpolationTypeToLinear();
	_volumeProperty->ShadeOn();
	_volumeProperty->DisableGradientOpacityOn();	
	_volumeProperty->SetColor(_ctfun);
	_volumeProperty->SetScalarOpacity(_tfun );

	_newvol = vtkVolume::New();
	_newvol->SetMapper(_volumeMapper );
	_newvol->SetProperty(_volumeProperty );
	_volumeMapper->SetInput( _vol );
	_volumeMapper->Update();
	_newvol->Update();
}


wxMaracasMultipleVolumeRendererManagerData::~wxMaracasMultipleVolumeRendererManagerData(){
	
	_tfun->Delete();
	_ctfun->Delete();
	_volumePlanes->Delete();
	_compositeFunction->Delete();
	_volumeMapper->Delete();
	_volumeProperty->Delete();
	_newvol->Delete();
	
}
/**
**	Set Volume Color
**/
void wxMaracasMultipleVolumeRendererManagerData::setVolumeColor(std::vector<double>& greylevelcolors,
								std::vector<double>& red,
								std::vector<double>& green,
								std::vector<double>& blue)
{	
	
	_ctfun->RemoveAllPoints();
	for(int i = 0; i < greylevelcolors.size();i++){
		_ctfun->AddRGBPoint(greylevelcolors[i], red[i],green[i], blue[i]);
	//			std::cout<<"transfer color function "<<greylevelcolors[i]<<" "<<red[i]<<" "<<green[i]<<" "<<blue[i]<<std::endl;
	}		
	_newvol->Update();	
}
				
/**
**	Volume Opacity
**/
void wxMaracasMultipleVolumeRendererManagerData::setVolumeOpacity(std::vector<double> greylevel,std::vector<double> value){
		
	_tfun->RemoveAllPoints();
	for(int i = 0; i < greylevel.size();i++){
		_tfun->AddPoint(greylevel[i], value[i]);
//			std::cout<<"transfer function "<<greylevel[i]<<" "<< value[i]<<std::endl;
	}
    _newvol->Update();
}

/**
**	Check if the variables are setted correctly
**/
void wxMaracasMultipleVolumeRendererManagerData::checkInvariant(){

}
/**
**	get the prop3D 
**/
vtkProp3D* wxMaracasMultipleVolumeRendererManagerData::getProp3D(){
	return this->_newvol;
}
/**
**	return the id from the daat
**/
int wxMaracasMultipleVolumeRendererManagerData::getId(){
	return _id;
}
/**
**	set data id
**/
void wxMaracasMultipleVolumeRendererManagerData::setId(int propid){
	_id = propid;
}

/**
**	Get the filanme
**/
std::string wxMaracasMultipleVolumeRendererManagerData::getDataname(){
	return _dataname;
}
/**
** Set the filanme
**/
void wxMaracasMultipleVolumeRendererManagerData::setDataname(std::string dataname){
	_dataname = dataname;
}

