#ifndef wxMaracasMultipleVolumeRendererManagerData_H_
#define wxMaracasMultipleVolumeRendererManagerData_H_

#include <vtkVolumeRayCastCompositeFunction.h>
#include <vtkPlanes.h>
#include <vtkVolumeRayCastMapper.h>
#include <vtkVolumeProperty.h>
#include <vtkVolume.h>
#include <vtkPiecewiseFunction.h>
#include <vtkColorTransferFunction.h>
#include <vtkImageData.h>
#include <vtkProp3D.h>
#include <vtkRenderer.h>

#include <vector>


class wxMaracasMultipleVolumeRendererManagerData  {

public:
	wxMaracasMultipleVolumeRendererManagerData(vtkImageData* vol, std::string dataname="");	
	~wxMaracasMultipleVolumeRendererManagerData();		
	
	/**
	**	Check if the variables are setted correctly
	**/
	void checkInvariant();
	/**
	**	get the prop3D 
	**/
	vtkProp3D* getProp3D();
	/**
	**	return the id from the daat
	**/
	int getId();
	/**
	**	set data id
	**/
	void setId(int propid);
	/**
	**	Get the filanme
	**/
    std::string getDataname();
	/**
	** Set the filanme
	**/
    void setDataname(std::string dataname);	

	
	/**
	**	Set Volume Color
	**/
	void setVolumeColor(std::vector<double>& greylevel,
									std::vector<double>& red,
									std::vector<double>& green,
									std::vector<double>& blue);	

	/**
	**	Volume Opacity
	**/
	void setVolumeOpacity(std::vector<double> greylevel,std::vector<double> value);

	vtkPiecewiseFunction* GetTransferFunction(){
		return _tfun;
	}
	vtkColorTransferFunction* GetColorFunction(){
		return _ctfun;
	}
	
protected:
	/**
	 * Prop 3D (data actor)
	 */
	vtkImageData* _vol;	
	/**
	 *  Dataname given by the user (ex. filename) 
	 **/
	std::string _dataname;	
	
	
private:
	
	/*
	 * id of the data
	 */
	int _id;		

	vtkVolumeRayCastCompositeFunction	*_compositeFunction;
	vtkPlanes							*_volumePlanes;
	vtkVolumeRayCastMapper				*_volumeMapper;
	vtkVolumeProperty					*_volumeProperty;
	vtkVolume							*_newvol;
	vtkPiecewiseFunction* _tfun;
	vtkColorTransferFunction* _ctfun;

	
};

#endif /*wxMaracasMultipleVolumeRendererManagerData_H_*/
