/*=========================================================================

  Program:   wxMaracas
  Module:    $RCSfile: wxMaracasMultipleVolumeRendererView.cxx,v $
  Language:  C++
  Date:      $Date: 2009/07/20 10:06:12 $
  Version:   $Revision: 1.2 $

  Copyright: (c) 2002, 2003
  License:

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/



// EOF - wxMaracasMPR.cxx

#include "wxMaracasMultipleVolumeRendererView.h"


#include <wx/colordlg.h>
#include <wx/bmpbuttn.h>

#include <OpenImage.xpm>
#include <Color.xpm>

wxMaracasMultipleVolumeRendererView* wxMaracasMultipleVolumeRendererView::instance=NULL;

wxMaracasMultipleVolumeRendererView::wxMaracasMultipleVolumeRendererView( wxWindow* parent,std::string path)
: wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize){

	wxauimanager = new wxAuiManager(this);

	_path = path;

	volmanager = new wxMaracasMultipleVolumeRendererManager();
	std::string iconsdir = path;
	iconsdir+="/data/Icons";
	this->_toolb = new ToolBarMultipleVolumeRenderer(this,iconsdir);
	wxAuiPaneInfo paneinfo;
	wxauimanager->AddPane(_toolb,paneinfo.ToolbarPane().Top());

	wxauimanager->Update();	

}
wxMaracasMultipleVolumeRendererView::~wxMaracasMultipleVolumeRendererView( ){

	delete _toolb;

}

std::string wxMaracasMultipleVolumeRendererView::getPath(){
	return _path;
}

wxMaracasMultipleVolumeRendererView* wxMaracasMultipleVolumeRendererView::getInstance(wxWindow* parent,std::string path){
	if(instance==NULL){
		instance = new wxMaracasMultipleVolumeRendererView(parent,path);
	}
	return instance;
}

wxMaracasMultipleVolumeRendererView* wxMaracasMultipleVolumeRendererView::getInstance(){
	return instance;
}

void wxMaracasMultipleVolumeRendererView::setRenderer(vtkRenderer*  renderer){
	volmanager->setRenderer(renderer);
}

void wxMaracasMultipleVolumeRendererView::addRemoveActor(int id, bool addremove){
	volmanager->addRemoveActor(id, addremove);
}

void wxMaracasMultipleVolumeRendererView::onLoadImageFile(){

	wxString mhd(_T("mhd"));	

	wxFileDialog* fildial = new wxFileDialog(this, wxString(_T("Select a MHD file")),wxString(_T("")),
		wxString(_T("")),wxString(_T("MHD files (*.mhd)|*.mhd")) );

	if(fildial->ShowModal()==wxID_OK){
        wxString filename = fildial->GetFilename();
		wxString pathfile(fildial->GetDirectory() + _T("/") + filename);
		if(filename.EndsWith(mhd)){
			loadVolume(pathfile,filename);
		}
	}
	delete fildial;

}

void wxMaracasMultipleVolumeRendererView::addVolumeViewPanel(wxMaracasMultipleVolumeRendererPanel* irmview, std::string dataname){

	wxString s(dataname.c_str(),wxConvUTF8 );
	wxAuiPaneInfo paneinfo;
	wxauimanager->AddPane(irmview, paneinfo.DefaultPane().Centre().DestroyOnClose().Caption(s));
	wxauimanager->Update();


}
	
void wxMaracasMultipleVolumeRendererView::addVolume(vtkImageData* img, std::string dataname){

	try{
		int id = volmanager->addVolume(img,dataname);		
		if(id!=-1){
			wxMaracasMultipleVolumeRendererPanel* controlpan = new wxMaracasMultipleVolumeRendererPanel(this, id,img);
			addVolumeViewPanel(controlpan, dataname);
			controlpan->updateVolume();
		}
	}catch(char* str){
		std::cout << "Exception : " << str << '\n';
		wxMessageDialog* diag = new wxMessageDialog(this, wxString(str,wxConvUTF8 ), wxString(str,wxConvUTF8 ), wxICON_ERROR);
		diag->ShowModal();
	}
}

void wxMaracasMultipleVolumeRendererView::loadVolume(wxString filename, wxString dataname){

	std::string s = std::string(filename.mb_str());
	vtkImageData* img = volmanager->getImageData(s);
	if(img!=NULL){
		s = std::string(dataname.mb_str());
		addVolume(img, s);
	}
    
}

void wxMaracasMultipleVolumeRendererView::deleteVolume(int volid){
	volmanager->deleteActor(volid);
}

void wxMaracasMultipleVolumeRendererView::SetValuesColorPointsFunction(int volid, std::vector<double> greylevelcolors,std::vector<double> red,std::vector<double> green,std::vector<double> blue){
	volmanager->setVolumeColor(volid, greylevelcolors, red, green, blue);
}
void wxMaracasMultipleVolumeRendererView::SetValuesPointsFunction(int volid, std::vector<double> greylevel, std::vector<double> values){
	volmanager->setVolumeOpacity(volid, greylevel, values);
}

vtkPiecewiseFunction* wxMaracasMultipleVolumeRendererView::GetTransferFunction(int volumeid){
	return volmanager->GetTransferFunction(volumeid);
}
vtkColorTransferFunction* wxMaracasMultipleVolumeRendererView::GetColorFunction(int volumeid){
    return volmanager->GetColorFunction(volumeid);
}
/**
**
**/

ToolBarMultipleVolumeRenderer::ToolBarMultipleVolumeRenderer(wxWindow * parent,std::string iconsdir)
: wxToolBar(parent, -1, wxDefaultPosition, wxDefaultSize)
{


	std::string iconfil = iconsdir;

	//iconfil+= "/OpenImage.png";
	//wxBitmap* bitmap0 = new wxBitmap(wxString(iconfil.c_str(),wxConvUTF8), wxBITMAP_TYPE_PNG);
	wxBitmap bitmap0(OpenImage_xpm);
	this->AddTool(1, wxString(_T("test")),bitmap0);

	/*iconfil+= "/Open.png";
	wxBitmap* bitmap2 = new wxBitmap(wxString(iconfil.c_str(),wxConvUTF8), wxBITMAP_TYPE_PNG);
	this->AddTool(2, wxString(_T("test")),*bitmap2);	*/

	/*iconfil = iconsdir;
	iconfil+= "/Open.png";
	wxBitmap* bitmap30 = new wxBitmap(wxString(iconfil.c_str(),wxConvUTF8), wxBITMAP_TYPE_PNG);
	this->AddTool(30, wxString(_T("test")),*bitmap30);*/

	this->Realize();

	_evthand = new ToolBarEventHandlerMultipleVolumeRenderer();
	this->SetEventHandler(_evthand);

}

ToolBarMultipleVolumeRenderer::~ToolBarMultipleVolumeRenderer(void){
}

ToolBarEventHandlerMultipleVolumeRenderer::ToolBarEventHandlerMultipleVolumeRenderer()
: wxEvtHandler(){
}
ToolBarEventHandlerMultipleVolumeRenderer::~ToolBarEventHandlerMultipleVolumeRenderer(){
}

void ToolBarEventHandlerMultipleVolumeRenderer::onLoadImageFile(wxCommandEvent& event){
	wxMaracasMultipleVolumeRendererView::getInstance()->onLoadImageFile();
}



BEGIN_EVENT_TABLE(ToolBarEventHandlerMultipleVolumeRenderer, wxEvtHandler)
	EVT_MENU(1, ToolBarEventHandlerMultipleVolumeRenderer::onLoadImageFile)
END_EVENT_TABLE()


