

#ifndef __VTK__STL__EXTRACTOR__HXX__
#define __VTK__STL__EXTRACTOR__HXX__


/*
*  La clase vtkSTLExtractor recibe como parametro un volumen binario y a partir de el
*  genera las superficies de estereolitografia interna y externa.
*  
*	The class vtkSTLExtractor receives a a binary volume and from this volume,
*	generates the inner and outer stereolitography surfaces.
*
*	Author: Diego Cantor.
*/


#include <vtkImageData.h>
#include <vtkPolyData.h>


class vtkSTLExtractor  
{
public:
	vtkSTLExtractor();
	virtual ~vtkSTLExtractor();
	void setVolume(vtkImageData *volume);
	void setMarchingCubesLevel(double level);
	void setSigmaLevel(double level);
	double getMarchingCubesLevel();
	double getSigmaLevel();
	void calculate();
	vtkPolyData* getInnerSurface();
	vtkPolyData* getOuterSurface();
	void Delete();
	
private:

	double sigmaLevel;
	double marchingCubesLevel;
	vtkPolyData *innerSurface;
	vtkPolyData *outerSurface;
	vtkImageData *volume;


};

#endif  //__VTK__STL__EXTRACTOR__HXX__
